@@ -21,8 +21,9 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#include <aconf.h>
+
 #include <gpdf-g-switch.h>
-#  include <gdk-pixbuf/gdk-pixbuf.h>
 #  include <gtk/gtk.h>
 #  include <libgnome/gnome-i18n.h>
 #  include <libgnomeui/gnome-app-helper.h>
@@ -31,7 +32,6 @@
 #  include "gpdf-marshal.h"
 #include <gpdf-g-switch.h>
 
-#include <aconf.h>
 #include "goo/GList.h"
 #include "Object.h"
 #include "Catalog.h"
@@ -68,20 +68,6 @@ struct _GPdfBookmarksViewPrivate {
 	gulong collapse_id;
 	gulong selection_id;
 
-	GdkPixbuf *stock_book_closed_pixbuf;
-	GdkPixbuf *stock_book_opened_pixbuf;
-	GdkPixbuf *stock_book_closed_mark_pixbuf;
-	GdkPixbuf *stock_book_opened_mark_pixbuf;
-	GdkPixbuf *stock_bookmarks_pixbuf;
-
-	GdkPixbuf *stock_first_pixbuf; 
-	GdkPixbuf *stock_last_pixbuf; 
-	GdkPixbuf *stock_prev_pixbuf; 
-	GdkPixbuf *stock_next_pixbuf; 
-	GdkPixbuf *stock_close_pixbuf; 
-	GdkPixbuf *stock_quit_pixbuf;
-	GdkPixbuf *stock_net_pixbuf;
-
 	GnomeUIInfo *popup_menu_uiinfo;
 	GtkWidget *popup_menu;
 
@@ -104,10 +90,6 @@ static void	gpdf_bookmarks_view_class_init	      (GPdfBookmarksViewClass*);
 static void	gpdf_bookmarks_view_dispose	      (GObject*); 
 static void	gpdf_bookmarks_view_finalize	      (GObject*); 
 static void	gpdf_bookmarks_view_instance_init     (GPdfBookmarksView*);
-static void     gpdf_bookmarks_view_row_expanded_cb   (GtkTreeView*, GtkTreeIter*,
-						       GtkTreePath*, gpointer); 
-static void     gpdf_bookmarks_view_row_collapsed_cb  (GtkTreeView*, GtkTreeIter*,
-						       GtkTreePath*, gpointer);
 static void	gpdf_bookmarks_view_page_changed_cb   (GPdfView *gpdf_view,
 						       int 	 page,
 						       gpointer  user_data);
@@ -124,11 +106,10 @@ static void	gpdf_bookmarks_view_popup_menu_item_collapse_all_cb (GtkMenuItem*,
 		(((obj) != NULL) && (GPDF_IS_BOOKMARKS_VIEW ((obj))))
 
 enum {
-	GPDF_BKVIEW_COLUMN1 = 0, 
-	GPDF_BKVIEW_COLUMN2, 
-	GPDF_BKVIEW_COLUMN3,
-	GPDF_BKVIEW_COLUMN4,
-	GPDF_BKVIEW_COLUMN5,
+	GPDF_BKVIEW_COLUMN_MARKUP, 
+	GPDF_BKVIEW_COLUMN_OUTLINE,
+	GPDF_BKVIEW_COLUMN_PAGE_NUM,
+	GPDF_BKVIEW_COLUMN_PAGE_VALID,
 	NUM_COLUMNS
 };
 
@@ -144,14 +125,14 @@ enum {
 	PROP_CONTROL
 };
 
-#define POPUP_MENU_EXPAND_ITEM			"Expand"
-#define POPUP_MENU_EXPAND_ITEM_TIP		"Expand all bookmarks under this one"
-#define POPUP_MENU_COLLAPSE_ITEM		"Collapse"
-#define POPUP_MENU_COLLAPSE_ITEM_TIP		"Collapse all bookmarks under this one"
-#define POPUP_MENU_EXPAND_ALL_ITEM		"Expand All"
-#define POPUP_MENU_EXPAND_ALL_ITEM_TIP		"Expand All bookmarks in bookmarks tree"
-#define POPUP_MENU_COLLAPSE_ALL_ITEM		"Collapse All"
-#define POPUP_MENU_COLLAPSE_ALL_ITEM_TIP	"Collapse All bookmarks in bookmarks tree"
+#define POPUP_MENU_EXPAND_ITEM			N_("Expand")
+#define POPUP_MENU_EXPAND_ITEM_TIP		N_("Expand all bookmarks under this one")
+#define POPUP_MENU_COLLAPSE_ITEM		N_("Collapse")
+#define POPUP_MENU_COLLAPSE_ITEM_TIP		N_("Collapse all bookmarks under this one")
+#define POPUP_MENU_EXPAND_ALL_ITEM		N_("Expand All")
+#define POPUP_MENU_EXPAND_ALL_ITEM_TIP		N_("Expand All bookmarks in bookmarks tree")
+#define POPUP_MENU_COLLAPSE_ALL_ITEM		N_("Collapse All")
+#define POPUP_MENU_COLLAPSE_ALL_ITEM_TIP	N_("Collapse All bookmarks in bookmarks tree")
 
 #define POPUP_MENU_EXPAND_INDEX			0
 #define POPUP_MENU_COLLAPSE_INDEX		1
@@ -160,17 +141,17 @@ enum {
 
 static GnomeUIInfo tools_popup_menu_items_init[] =
 {
-      GNOMEUIINFO_ITEM_NONE (N_ (POPUP_MENU_EXPAND_ITEM),
-			     N_ (POPUP_MENU_EXPAND_ITEM_TIP),
+      GNOMEUIINFO_ITEM_NONE (_(POPUP_MENU_EXPAND_ITEM),
+			     _(POPUP_MENU_EXPAND_ITEM_TIP),
 			     gpdf_bookmarks_view_popup_menu_item_expand_cb),
-      GNOMEUIINFO_ITEM_NONE (N_ (POPUP_MENU_COLLAPSE_ITEM),
-			     N_ (POPUP_MENU_COLLAPSE_ITEM_TIP),
+      GNOMEUIINFO_ITEM_NONE (_(POPUP_MENU_COLLAPSE_ITEM),
+			     _(POPUP_MENU_COLLAPSE_ITEM_TIP),
 			     gpdf_bookmarks_view_popup_menu_item_collapse_cb),
-      GNOMEUIINFO_ITEM_NONE (N_ (POPUP_MENU_EXPAND_ALL_ITEM),
-			     N_ (POPUP_MENU_EXPAND_ALL_ITEM_TIP),
+      GNOMEUIINFO_ITEM_NONE (_(POPUP_MENU_EXPAND_ALL_ITEM),
+			     _(POPUP_MENU_EXPAND_ALL_ITEM_TIP),
 			     gpdf_bookmarks_view_popup_menu_item_expand_all_cb),
-      GNOMEUIINFO_ITEM_NONE (N_ (POPUP_MENU_COLLAPSE_ALL_ITEM),
-			     N_ (POPUP_MENU_COLLAPSE_ALL_ITEM_TIP),
+      GNOMEUIINFO_ITEM_NONE (_(POPUP_MENU_COLLAPSE_ALL_ITEM),
+			     _(POPUP_MENU_COLLAPSE_ALL_ITEM_TIP),
 			     gpdf_bookmarks_view_popup_menu_item_collapse_all_cb), 
       GNOMEUIINFO_END
 };
@@ -312,169 +293,13 @@ gpdf_bookmarks_view_construct (GPdfBookmarksView *bookmarks_view)
 
 	/* Create tree view */
 	priv->treeview = gtk_tree_view_new ();
+	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (priv->treeview), FALSE);
+
 	gtk_widget_show (priv->treeview);
 	gtk_container_add (GTK_CONTAINER (bookmarks_view), priv->treeview);
 	
-	priv->expand_id =
-	  g_signal_connect (G_OBJECT (priv->treeview), "row_expanded",
-			    G_CALLBACK (gpdf_bookmarks_view_row_expanded_cb),
-			    bookmarks_view);
-
-	priv->collapse_id =
-	  g_signal_connect (G_OBJECT (priv->treeview), "row_collapsed",
-			    G_CALLBACK (gpdf_bookmarks_view_row_collapsed_cb),
-			    bookmarks_view);
-	
 	bkwidget = GTK_WIDGET (bookmarks_view);
 
-	/* Toolbar bookmarks icon */
-	GPDF_RENDER_STOCK_ICON(bkwidget,
-			       priv->stock_bookmarks_pixbuf, 
-			       GPDF_STOCK_BOOKMARKS,
-			       GTK_ICON_SIZE_SMALL_TOOLBAR); 
-
-	/* Tree closed book w/o mark */
-	GPDF_RENDER_STOCK_ICON(bkwidget,
-			       priv->stock_book_closed_pixbuf, 
-			       GPDF_STOCK_BOOK_CLOSED,
-			       GTK_ICON_SIZE_SMALL_TOOLBAR); 
-	/* Tree opened book w/o mark */
-	GPDF_RENDER_STOCK_ICON(bkwidget,
-			       priv->stock_book_opened_pixbuf, 
-			       GPDF_STOCK_BOOK_OPENED,
-			       GTK_ICON_SIZE_SMALL_TOOLBAR);
-
-	/* Tree closed book w/ mark */
-	GPDF_RENDER_STOCK_ICON(bkwidget,
-			       priv->stock_book_closed_mark_pixbuf, 
-			       GPDF_STOCK_BOOK_CLOSED_MARK,
-			       GTK_ICON_SIZE_SMALL_TOOLBAR); 
-	/* Tree opened book w/ mark */
-	GPDF_RENDER_STOCK_ICON(bkwidget,
-			       priv->stock_book_opened_mark_pixbuf, 
-			       GPDF_STOCK_BOOK_OPENED_MARK,
-			       GTK_ICON_SIZE_SMALL_TOOLBAR);
-
-	/* Stock first */
-	GPDF_RENDER_STOCK_ICON(bkwidget,
-			       priv->stock_first_pixbuf, 
-			       GTK_STOCK_GOTO_FIRST,
-			       GTK_ICON_SIZE_SMALL_TOOLBAR);
-	/* Stock last */
-	GPDF_RENDER_STOCK_ICON(bkwidget,
-			       priv->stock_last_pixbuf, 
-			       GTK_STOCK_GOTO_LAST,
-			       GTK_ICON_SIZE_SMALL_TOOLBAR); 
-
-	/* Stock previous */
-	GPDF_RENDER_STOCK_ICON(bkwidget,
-			       priv->stock_prev_pixbuf, 
-			       GTK_STOCK_GO_BACK,
-			       GTK_ICON_SIZE_SMALL_TOOLBAR); 
-	/* Stock next */
-	GPDF_RENDER_STOCK_ICON(bkwidget,
-			       priv->stock_next_pixbuf, 
-			       GTK_STOCK_GO_FORWARD,
-			       GTK_ICON_SIZE_SMALL_TOOLBAR); 
-
-	/* Stock close */
-	GPDF_RENDER_STOCK_ICON(bkwidget,
-			       priv->stock_close_pixbuf, 
-			       GTK_STOCK_CLOSE,
-			       GTK_ICON_SIZE_SMALL_TOOLBAR); 
-
-	/* Stock quit */
-	GPDF_RENDER_STOCK_ICON(bkwidget,
-			       priv->stock_quit_pixbuf, 
-			       GTK_STOCK_QUIT,
-			       GTK_ICON_SIZE_SMALL_TOOLBAR);
-
-	/* Stock URI */
-	GPDF_RENDER_STOCK_ICON(bkwidget,
-			       priv->stock_net_pixbuf, 
-			       GTK_STOCK_NETWORK,
-			       GTK_ICON_SIZE_SMALL_TOOLBAR);
-
-}
-
-static void
-gpdf_bookmarks_view_row_expanded_cb (GtkTreeView *treeview,
-				     GtkTreeIter *iter,
-				     GtkTreePath *path,
-				     gpointer user_data)
-{
-	GPdfBookmarksView *bookmarks_view = GPDF_BOOKMARKS_VIEW (user_data);
-	GPdfBookmarksViewPrivate *priv;
-	GValue page_nr_value = {0, };
-	gint page_nr; 
-	GdkPixbuf *bookmark_pixbuf = NULL;
-	gboolean page_displayed; 
-	
-	g_return_if_fail (GPDF_IS_NON_NULL_BOOKMARKS_VIEW (bookmarks_view));
-	priv = bookmarks_view->priv;
-
-	if (!priv->generation_terminated) return; 
-
-	/* Get bookmark page nr */
-	gtk_tree_model_get_value (GTK_TREE_MODEL(priv->model),
-				  iter,
-				  GPDF_BKVIEW_COLUMN5,
-				  &page_nr_value);
-	page_nr = g_value_get_int ((const GValue *)&page_nr_value);
-
-	/* Is its page displayed */
-	page_displayed = (page_nr == priv->current_page);
-	
-	/* Change appearance accordingly */
-	if (page_displayed)
-		bookmark_pixbuf = priv->stock_book_opened_mark_pixbuf; 
-	else
-		bookmark_pixbuf = priv->stock_book_opened_pixbuf;
-	
-	gtk_tree_store_set (GTK_TREE_STORE (priv->model), iter,
-			    GPDF_BKVIEW_COLUMN1, bookmark_pixbuf,
-			    -1);
-	
-}
-
-static void
-gpdf_bookmarks_view_row_collapsed_cb (GtkTreeView *treeview,
-				      GtkTreeIter *iter,
-				      GtkTreePath *path,
-				      gpointer user_data)
-{
-	GPdfBookmarksView *bookmarks_view = GPDF_BOOKMARKS_VIEW (user_data);
-	GPdfBookmarksViewPrivate *priv; 
-	GValue page_nr_value = {0, };
-	gint page_nr; 
-	GdkPixbuf *bookmark_pixbuf = NULL;
-	gboolean page_displayed; 
-	
-	g_return_if_fail (GPDF_IS_NON_NULL_BOOKMARKS_VIEW (bookmarks_view));
-	priv = bookmarks_view->priv; 
-
-	if (!priv->generation_terminated) return; 
-
-	/* Get bookmark page nr */
-	gtk_tree_model_get_value (GTK_TREE_MODEL (priv->model),
-				  iter,
-				  GPDF_BKVIEW_COLUMN5,
-				  &page_nr_value);
-	page_nr = g_value_get_int ((const GValue *)&page_nr_value);
-
-	/* Is its page displayed */
-	page_displayed = (page_nr == priv->current_page);
-
-	/* Change appearance accordingly */
-	if (page_displayed)
-		bookmark_pixbuf = priv->stock_book_closed_mark_pixbuf; 
-	else
-		bookmark_pixbuf = priv->stock_book_closed_pixbuf; 
-
-	gtk_tree_store_set (GTK_TREE_STORE (priv->model), iter,
-			    GPDF_BKVIEW_COLUMN1, bookmark_pixbuf,
-			    -1);
-	
 }
 
 gboolean
@@ -489,7 +314,6 @@ gpdf_bookmarks_view_update_bkvisual (GtkTreeModel *model,
 	OutlineItem *anItem;
 	GValue an_item_value = {0, };
 	gint page_nr;
-	GdkPixbuf *bookmark_pixbuf = NULL;
 	gboolean page_displayed, is_expanded; 
 	
 	bookmarks_view = GPDF_BOOKMARKS_VIEW (data);
@@ -502,7 +326,7 @@ gpdf_bookmarks_view_update_bkvisual (GtkTreeModel *model,
 	/* Get item to access action kind */
 	gtk_tree_model_get_value (model,
 				  iter,
-				  GPDF_BKVIEW_COLUMN4,
+				  GPDF_BKVIEW_COLUMN_OUTLINE,
 				  &an_item_value);
 	anItem = (OutlineItem *) g_value_get_pointer ((const GValue *)&an_item_value);
 
@@ -513,7 +337,7 @@ gpdf_bookmarks_view_update_bkvisual (GtkTreeModel *model,
 		/* Get this bookmark page nr */
 		gtk_tree_model_get_value (model,
 					  iter,
-					  GPDF_BKVIEW_COLUMN5,
+					  GPDF_BKVIEW_COLUMN_PAGE_NUM,
 					  &page_nr_value);
 		page_nr = g_value_get_int ((const GValue *)&page_nr_value);
 		
@@ -522,23 +346,6 @@ gpdf_bookmarks_view_update_bkvisual (GtkTreeModel *model,
 		is_expanded = (gtk_tree_view_row_expanded (GTK_TREE_VIEW (priv->treeview),
 							   path));
 		
-		/* And take pixbuf accordingly to set in store */
-		if (page_displayed) {
-			if (is_expanded)
-			  bookmark_pixbuf = priv->stock_book_opened_mark_pixbuf; 
-			else
-			  bookmark_pixbuf = priv->stock_book_closed_mark_pixbuf; 
-		}
-		else {
-			if (is_expanded)
-			  bookmark_pixbuf = priv->stock_book_opened_pixbuf; 
-			else
-			  bookmark_pixbuf = priv->stock_book_closed_pixbuf; 
-		}
-		gtk_tree_store_set (GTK_TREE_STORE (model), iter,
-				    GPDF_BKVIEW_COLUMN1, bookmark_pixbuf,
-				    -1);
-
 		/* For convinience, expand and scroll to current page bookmarks */
 		if (page_displayed)
 		{
@@ -551,7 +358,7 @@ gpdf_bookmarks_view_update_bkvisual (GtkTreeModel *model,
 				path,
 				gtk_tree_view_get_column(
 					GTK_TREE_VIEW (priv->treeview),
-					GPDF_BKVIEW_COLUMN2),
+					GPDF_BKVIEW_COLUMN_MARKUP),
 				TRUE, 0.0, 0.0); 
 		}
 	}
@@ -765,58 +572,10 @@ gpdf_bookmarks_view_update_popup_actions (GPdfBookmarksView *bookmarks_view)
 	}
 }
 
-static gboolean
-gpdf_bookmarks_view_bookmark_select_func (GtkTreeSelection  *selection,
-					  GtkTreeModel	    *model,
-					  GtkTreePath	    *path,
-					  gboolean	     path_currently_selected,
-					  gpointer	     data)
-{
-	GPdfBookmarksView *bookmarks_view;
-	GPdfBookmarksViewPrivate *priv;	
-	GValue selection_item = {0,};
-	GtkTreeIter iterator;
-	OutlineItem *item;
-	gboolean ret = FALSE;
-
-	bookmarks_view = GPDF_BOOKMARKS_VIEW (data);
-	
-	g_return_val_if_fail (GPDF_IS_NON_NULL_BOOKMARKS_VIEW (bookmarks_view),
-			      FALSE);
-
-	do {
-		priv = bookmarks_view->priv;
-
-		gtk_tree_model_get_iter (GTK_TREE_MODEL (priv->model),
-					 &iterator,
-					 path);
-
-		/* handle only if is about to be selected */
-		if (!gtk_tree_selection_get_selected (selection, 
-						      &model,
-						      &iterator)) {
-			gtk_tree_model_get_value (GTK_TREE_MODEL (priv->model),
-						  &iterator,
-						  GPDF_BKVIEW_COLUMN4,
-						  &selection_item);
-			item = (OutlineItem *)g_value_peek_pointer ((const GValue*) &selection_item);
-
-			gpdf_bookmarks_view_emit_bookmark_selected (bookmarks_view,
-								    item ? item->getAction () : NULL);
-		}
-		
-		ret = TRUE;
-	}
-	while (0); 
-
-	return ret;
-}
-
 static void
 gpdf_bookmarks_view_flat_recurse_outlines (GPdfBookmarksView *bookmarks_view)
 {
 	GPdfBookmarksViewPrivate *priv;
-	GdkPixbuf *bookmark_pixbuf = NULL; 
 	LinkAction *link_action = NULL;
 	LinkNamed *link_named = NULL;
 	LinkGoTo *link_goto = NULL;
@@ -826,10 +585,10 @@ gpdf_bookmarks_view_flat_recurse_outlines (GPdfBookmarksView *bookmarks_view)
 	Ref page_ref;
 	gchar *action_name; 
 	int i, page = 0;
-	gchar *pagestr = NULL;
 	gboolean is_expanded; 
 	gchar *status = NULL;
-	double ratio = 0.0; 
+	double ratio = 0.0;
+	gboolean page_valid = FALSE;
 	
 	g_return_if_fail (GPDF_IS_NON_NULL_BOOKMARKS_VIEW (bookmarks_view)); 
 
@@ -840,7 +599,8 @@ gpdf_bookmarks_view_flat_recurse_outlines (GPdfBookmarksView *bookmarks_view)
 	{
 		GtkTreeIter child_iter;
 		OutlineItem *anItem;
-		Unicode *title; 
+		Unicode *title;
+		gchar *title_text;
 		
 		anItem = (OutlineItem *)priv->generation_state->items->get(i);
 		priv->generation_state->i++;
@@ -864,7 +624,9 @@ gpdf_bookmarks_view_flat_recurse_outlines (GPdfBookmarksView *bookmarks_view)
 			case actionGoTo:
 				link_goto = dynamic_cast <LinkGoTo *> (link_action);
 				link_dest = link_goto->getDest ();
-				named_dest = link_goto->getNamedDest (); 
+				named_dest = link_goto->getNamedDest ();
+				page_valid = TRUE;
+
 				if (link_dest != NULL) {
 					link_dest = link_dest->copy ();
 				} else if (named_dest != NULL) {
@@ -882,20 +644,7 @@ gpdf_bookmarks_view_flat_recurse_outlines (GPdfBookmarksView *bookmarks_view)
 					
 					delete link_dest;
 				}
-				pagestr = g_strdup_printf ("%d", page); 
 				
-				if (page == priv->current_page) {
-					if (is_expanded)
-						bookmark_pixbuf = priv->stock_book_opened_mark_pixbuf;
-					else
-						bookmark_pixbuf = priv->stock_book_closed_mark_pixbuf;
-				}
-				else {
-					if (is_expanded)
-						bookmark_pixbuf = priv->stock_book_opened_pixbuf;
-					else
-						bookmark_pixbuf = priv->stock_book_closed_pixbuf;
-				}
 				break;
 				
 			case actionNamed:
@@ -903,40 +652,28 @@ gpdf_bookmarks_view_flat_recurse_outlines (GPdfBookmarksView *bookmarks_view)
 				action_name = link_named->getName ()->getCString ();
 				if (strcmp (action_name, "NextPage") == 0) {
 					page = priv->current_page +1;
-					bookmark_pixbuf = priv->stock_next_pixbuf; 
 				} else if (strcmp (action_name, "PrevPage") == 0) {
 					page = priv->current_page -1; 
-					bookmark_pixbuf = priv->stock_prev_pixbuf; 
 				} else if (strcmp (action_name, "FirstPage") == 0) {
 					page = 1; 
-					bookmark_pixbuf = priv->stock_first_pixbuf; 
 				} else if (strcmp (action_name, "LastPage") == 0) {
 					page = priv->pdf_doc->getNumPages (); 
-					bookmark_pixbuf = priv->stock_last_pixbuf; 
 				} else if (strcmp (action_name, "Close") == 0) {
 					page = -1; 
-					bookmark_pixbuf = priv->stock_close_pixbuf; 
 				} else if (strcmp (action_name, "Quit") == 0) {
 					page = -1; 
-					bookmark_pixbuf = priv->stock_quit_pixbuf; 
 				} else {
 					g_warning ("Unimplemented named link action: %s", action_name);
-					pagestr = g_strdup_printf (_("** Unknown %s **"), action_name); 
 				}
-				if (page == -1)
-					pagestr = g_strdup (action_name); 
-				else if (page > 0)
-					pagestr = g_strdup_printf ("%d", page);
+				if (page >= 0)
+					page_valid = TRUE;
 				break;
 
 			case actionURI:
 				link_uri = dynamic_cast <LinkURI *> (link_action);
-				pagestr = g_strdup (link_uri->getURI ()->getCString ());
-				bookmark_pixbuf = priv->stock_net_pixbuf;
 				break;
 
 			default:
-				pagestr = g_strdup ("** Unknown action **"); 
 				g_warning ("Unkown link action type: %d", link_action->getKind ());
 			}
 
@@ -948,14 +685,19 @@ gpdf_bookmarks_view_flat_recurse_outlines (GPdfBookmarksView *bookmarks_view)
 			gpdf_control_private_set_status (priv->parent, status);
 			gpdf_control_private_set_fraction (priv->parent, ratio); 
 			g_free (status);
-			
+			if (link_action->getKind () == actionURI) {
+				title_text = g_strdup_printf ("<span foreground=\"blue\" underline=\"single\">%s</span>",
+							      unicode_to_char (anItem, priv->umap));
+			} else {
+				title_text = g_strdup (unicode_to_char (anItem, priv->umap));
+			}
 			gtk_tree_store_set (GTK_TREE_STORE (priv->model), &child_iter,
-					    GPDF_BKVIEW_COLUMN1, bookmark_pixbuf, 
-					    GPDF_BKVIEW_COLUMN2, unicode_to_char (anItem, priv->umap), 
-					    GPDF_BKVIEW_COLUMN3, pagestr,
-					    GPDF_BKVIEW_COLUMN4, (gpointer)anItem,
-					    GPDF_BKVIEW_COLUMN5, page, 
+					    GPDF_BKVIEW_COLUMN_MARKUP, title_text, 
+					    GPDF_BKVIEW_COLUMN_OUTLINE, (gpointer)anItem,
+					    GPDF_BKVIEW_COLUMN_PAGE_NUM, page,
+					    GPDF_BKVIEW_COLUMN_PAGE_VALID, page_valid,
 					    -1);
+			g_free (title_text);
 		}
 		else if (title) {
 			gtk_tree_store_append (GTK_TREE_STORE (priv->model),
@@ -969,17 +711,11 @@ gpdf_bookmarks_view_flat_recurse_outlines (GPdfBookmarksView *bookmarks_view)
 						GTK_TREE_MODEL (priv->model),
 						&child_iter)));
 
-			if (is_expanded)
-				bookmark_pixbuf = priv->stock_book_opened_mark_pixbuf;
-			else
-				bookmark_pixbuf = priv->stock_book_closed_mark_pixbuf;
-			
 			gtk_tree_store_set (GTK_TREE_STORE (priv->model), &child_iter,
-					    GPDF_BKVIEW_COLUMN1, bookmark_pixbuf, 
-					    GPDF_BKVIEW_COLUMN2, unicode_to_char (anItem, priv->umap), 
-					    GPDF_BKVIEW_COLUMN3, "0",
-					    GPDF_BKVIEW_COLUMN4, (gpointer)NULL,
-					    GPDF_BKVIEW_COLUMN5, 0, 
+					    GPDF_BKVIEW_COLUMN_MARKUP, unicode_to_char (anItem, priv->umap), 
+					    GPDF_BKVIEW_COLUMN_OUTLINE, (gpointer)NULL,
+					    GPDF_BKVIEW_COLUMN_PAGE_NUM, 0,
+					    GPDF_BKVIEW_COLUMN_PAGE_VALID, FALSE,
 					    -1);
 		}
 		
@@ -1004,6 +740,35 @@ gpdf_bookmarks_view_flat_recurse_outlines (GPdfBookmarksView *bookmarks_view)
 	}
 }
 
+static void
+gpdf_bookmarks_page_num_func (GtkTreeViewColumn *tree_column,
+			      GtkCellRenderer   *cell,
+			      GtkTreeModel      *tree_model,
+			      GtkTreeIter       *iter,
+			      gpointer           data)
+{
+	int page_num;
+	gboolean page_valid;
+
+	gtk_tree_model_get (tree_model, iter,
+			    GPDF_BKVIEW_COLUMN_PAGE_NUM, &page_num,
+			    GPDF_BKVIEW_COLUMN_PAGE_VALID, &page_valid,
+			    -1);
+
+	if (page_valid) {
+		gchar *markup = g_strdup_printf ("<i>%d</i>", page_num);
+		g_object_set (cell,
+			      "markup", markup,
+			      "visible", TRUE,
+			      NULL);
+		g_free (markup);
+	} else {
+		g_object_set (cell,
+			      "visible", FALSE,
+			      NULL);
+	}
+}
+
 static void
 gpdf_bookmarks_view_update_bookmarks_tree (GPdfBookmarksView *bookmarks_view)
 {
@@ -1012,7 +777,6 @@ gpdf_bookmarks_view_update_bookmarks_tree (GPdfBookmarksView *bookmarks_view)
 	GString *enc; 
 	GtkCellRenderer *renderer;
 	GtkTreeViewColumn *column;
-	GtkTreeSelection *selection;
 	
 	g_return_if_fail (GPDF_IS_NON_NULL_BOOKMARKS_VIEW (bookmarks_view));
 	
@@ -1028,11 +792,10 @@ gpdf_bookmarks_view_update_bookmarks_tree (GPdfBookmarksView *bookmarks_view)
 		}
 	
 		priv->model = gtk_tree_store_new (NUM_COLUMNS,
-						  GDK_TYPE_PIXBUF,
-						  G_TYPE_STRING,
 						  G_TYPE_STRING,
 						  G_TYPE_POINTER,
-						  G_TYPE_INT);
+						  G_TYPE_INT,
+						  G_TYPE_BOOLEAN);
 
 		if (!priv->umap) {
 			enc = new GString("UTF-8");
@@ -1044,30 +807,26 @@ gpdf_bookmarks_view_update_bookmarks_tree (GPdfBookmarksView *bookmarks_view)
 		gtk_tree_view_set_model (GTK_TREE_VIEW (priv->treeview),
 					 GTK_TREE_MODEL (priv->model));
 		
-		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->treeview));
-		gtk_tree_selection_set_select_function (selection,
-							gpdf_bookmarks_view_bookmark_select_func,
-							bookmarks_view, NULL);
-		
 		gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (priv->treeview), TRUE);
 		
-		renderer = gtk_cell_renderer_pixbuf_new ();
-		column = gtk_tree_view_column_new_with_attributes (NULL, renderer,
-								   "pixbuf", GPDF_BKVIEW_COLUMN1, 
-								   NULL);
+		column = gtk_tree_view_column_new ();
+		gtk_tree_view_column_set_expand (GTK_TREE_VIEW_COLUMN (column), TRUE);
 		gtk_tree_view_append_column (GTK_TREE_VIEW (priv->treeview), column);
+
+		renderer = (GtkCellRenderer*)
+			g_object_new (GTK_TYPE_CELL_RENDERER_TEXT,
+				      "ellipsize", PANGO_ELLIPSIZE_END,
+				      NULL);
+		gtk_tree_view_column_pack_start (GTK_TREE_VIEW_COLUMN (column), renderer, TRUE);
+		gtk_tree_view_column_set_attributes (GTK_TREE_VIEW_COLUMN (column), renderer,
+						     "markup", GPDF_BKVIEW_COLUMN_MARKUP,
+						     NULL);
 		
 		renderer = gtk_cell_renderer_text_new ();
-		column = gtk_tree_view_column_new_with_attributes (_("Title"), renderer,
-								   "text", GPDF_BKVIEW_COLUMN2,
-								   NULL);
-		gtk_tree_view_append_column (GTK_TREE_VIEW (priv->treeview), column);
-		
-		renderer = gtk_cell_renderer_text_new ();
-		column = gtk_tree_view_column_new_with_attributes (_("Page"), renderer,
-								   "text", GPDF_BKVIEW_COLUMN3,
-								   NULL);
-		gtk_tree_view_append_column (GTK_TREE_VIEW (priv->treeview), column);
+		gtk_tree_view_column_pack_end (GTK_TREE_VIEW_COLUMN (column), renderer, FALSE);
+		gtk_tree_view_column_set_cell_data_func (GTK_TREE_VIEW_COLUMN (column), renderer,
+							 (GtkTreeCellDataFunc) gpdf_bookmarks_page_num_func,
+							 NULL, NULL);
 
 		priv->generation_state = g_new0 (GPdfBookmarksGenState, 1);
 		priv->generation_head = priv->generation_state; 
@@ -1086,12 +845,27 @@ static void
 gpdf_bookmarks_view_selection_changed_cb (GtkTreeSelection *selection, gpointer data)
 {
 	GPdfBookmarksView *bookmarks_view = GPDF_BOOKMARKS_VIEW (data);
+	GtkTreeIter iterator;
+	GtkTreeModel *model;
+	OutlineItem *item;
+	GValue selection_item = {0,};
 
 	g_return_if_fail (GPDF_IS_NON_NULL_BOOKMARKS_VIEW (bookmarks_view));
 	
 	if (!bookmarks_view->priv->generation_terminated) return;
 		
 	gpdf_bookmarks_view_update_popup_actions (bookmarks_view);
+
+	if (gtk_tree_selection_get_selected (selection, &model, &iterator)) {
+		gtk_tree_model_get_value (GTK_TREE_MODEL (model),
+					  &iterator,
+					  GPDF_BKVIEW_COLUMN_OUTLINE,
+					  &selection_item);
+		item = (OutlineItem *)g_value_peek_pointer ((const GValue*) &selection_item);
+
+		gpdf_bookmarks_view_emit_bookmark_selected (bookmarks_view,
+							    item ? item->getAction () : NULL);
+	}
 }
 
 static void
