@@ -337,8 +337,7 @@ static void
 gpdf_view_link_action_uri (GPdfView *gpdf_view, LinkURI *link_uri)
 {
 	gchar *uri; 
-	GnomeVFSMimeAction *action; 
-	GnomeVFSMimeApplication *app;
+	GnomeVFSMimeApplication *app = NULL;
 	GnomeVFSURI *vfsuri; 
 	G_List *uris = NULL;
 	const gchar *mime;
@@ -357,58 +356,24 @@ gpdf_view_link_action_uri (GPdfView *gpdf_view, LinkURI *link_uri)
 
 	g_message ("gpdf_view_link_action_uri: mime = '%s'", mime); 
 
-	if (!mime) {
-		mime = "text/html";
-		g_message ("gpdf_view_link_action_uri: using default mime = '%s'", mime); 
+	if (mime) {
+		app = gnome_vfs_mime_get_default_application (mime);
 	}
-
-	action = gnome_vfs_mime_get_default_action (mime); 
-	if (gnome_vfs_mime_action_launch (action, uris) != GNOME_VFS_OK) {
-		app = gnome_vfs_mime_get_default_application (mime); 
 		
-		if (!app || gnome_vfs_mime_application_launch (app, uris) != GNOME_VFS_OK) {
+	if (!app || gnome_vfs_mime_application_launch (app, uris) != GNOME_VFS_OK) {
+		gchar *msg;
 
-			/* Fallback: No component or app was found. Use a browser */
-			mime = "text/html";
-			action = gnome_vfs_mime_get_default_action (mime);
-			if (gnome_vfs_mime_action_launch (action, uris) != GNOME_VFS_OK) {
-				app = gnome_vfs_mime_get_default_application (mime); 
-				if (!app) {
-					gchar *msg = 
-					  g_strdup_printf(_("No application was found to open the URI: %s"),
-							  (const gchar *)uris->data);
-					gpdf_control_private_error_dialog (gpdf_view->priv->parent, 
-									   _("Application Launch Failure!"),
-									   msg,
-									   TRUE,
-									   FALSE);
-					g_free (msg);
-				} else if (gnome_vfs_mime_application_launch (app, uris) != GNOME_VFS_OK) {
-					gchar *msg = 
-					  g_strdup_printf(_("Failed to launch %s for URI: %s"),
-							  app->name,
-							  (const gchar *)uris->data);
-					gpdf_control_private_error_dialog (gpdf_view->priv->parent, 
-									   _("Application Launch Failure!"),
-									   msg,
-									   TRUE,
-									   FALSE);
-					g_free (msg);
-				}
-			}
-		} else {
-			gchar *msg = g_strdup_printf ("URI '%s' launched with %s",
-						      (const gchar *)uris->data,
-						      app->name);
-			gpdf_control_private_info_dialog (gpdf_view->priv->parent, 
-							  _("Application Launch notification!"),
-							  msg,
-							  TRUE);
-			g_free (msg);			
-		}
-
-		gnome_vfs_mime_application_free (app); 
+		msg = g_strdup_printf(_("No application was found to open the URI: %s"),
+				      (const gchar *)uris->data);
+		gpdf_control_private_error_dialog (gpdf_view->priv->parent, 
+						   _("Application Launch Failure!"),
+						   msg,
+						   TRUE,
+						   FALSE);
+		g_free (msg);
 	}
+	
+	gnome_vfs_mime_application_free (app); 
 	g_list_free (uris); 
 	gnome_vfs_uri_unref (vfsuri); 
 	delete uri; 
@@ -547,7 +512,7 @@ gpdf_view_link_entered_cb (GPdfLinksCanvasLayer *links_layer,
 		  if (filename)
 			  text = g_strdup_printf (_("Go to file %s:#%d"), filename->getCString (), page);
 		  else
-			  text = g_strdup_printf (_("Go to file %s:#%d"), "", page);
+			  text = g_strdup (_("Broken link"));
 		  break;
 	      }		
 	      case actionLaunch:
